@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -29,6 +24,7 @@ class CApiService {
 	public static $userData;
 
 	public const ACCESS_RULES = [];
+	public const OUTPUT_FIELDS = [];
 
 	/**
 	 * The name of the table.
@@ -162,10 +158,6 @@ class CApiService {
 		if ($tableName) {
 			$schema = $this->getTableSchema($tableName);
 
-			if (strpos($schema['key'], ',') !== false) {
-				throw new Exception('Composite private keys are not supported in this API version.');
-			}
-
 			return $schema['key'];
 		}
 
@@ -173,7 +165,7 @@ class CApiService {
 	}
 
 	/**
-	 * Returns the name of the option that refers the PK column. If the $tableName parameter
+	 * Returns the name of the option that refers to the PK column. If the $tableName parameter
 	 * is not given, the Pk option of the current table will be returned.
 	 *
 	 * @param string $tableName
@@ -271,15 +263,15 @@ class CApiService {
 	}
 
 	/**
-	 * Unsets fields $fields from the given objects if they are not requested in $output.
+	 * Unset those fields of the objects, which are not requested for the $output.
 	 *
 	 * @param array        $objects
 	 * @param array        $fields
-	 * @param string|array $output		desired output
+	 * @param string|array $output   requested output
 	 *
 	 * @return array
 	 */
-	protected function unsetExtraFields(array $objects, array $fields, $output) {
+	protected function unsetExtraFields(array $objects, array $fields, $output = []) {
 		// find the fields that have not been requested
 		$extraFields = [];
 		foreach ($fields as $field) {
@@ -375,7 +367,7 @@ class CApiService {
 	 * @param string $tableName
 	 * @param array  $options
 	 *
-	 * @return array
+	 * @return string
 	 */
 	protected function createSelectQuery($tableName, array $options) {
 		$sqlParts = $this->createSelectQueryParts($tableName, $this->tableAlias(), $options);
@@ -418,13 +410,17 @@ class CApiService {
 	}
 
 	/**
-	 * Returns DISTINCT modifier for sql statements with multiple joins.
+	 * Returns DISTINCT modifier for sql statements with multiple joins and without aggregations.
 	 *
 	 * @param array $sql_parts  An SQL parts array.
 	 *
 	 * @return string
 	 */
 	protected static function dbDistinct(array $sql_parts) {
+		if (preg_grep('/^COUNT\(/', $sql_parts['select'])) {
+			return '';
+		}
+
 		$count = count($sql_parts['from']);
 
 		if ($count == 1 && array_key_exists('left_join', $sql_parts)) {
@@ -432,7 +428,8 @@ class CApiService {
 				$r_table = DB::getSchema($left_join['table']);
 
 				// Increase count when table linked by non-unique column.
-				if ($left_join['using'] !== $r_table['key']) {
+				if ((array_key_exists('using', $left_join) && $left_join['using'] !== $r_table['key'])
+						|| (!array_key_exists('using', $left_join) && $left_join['use_distinct'])) {
 					$count++;
 					break;
 				}
@@ -455,9 +452,11 @@ class CApiService {
 			$l_table = DB::getSchema($sqlParts['left_table']['table']);
 
 			foreach ($sqlParts['left_join'] as $left_join) {
-				$sql_left_join .= ' LEFT JOIN '.$left_join['table'].' '.$left_join['alias'].
-					' ON '.$sqlParts['left_table']['alias'].'.'.$l_table['key'].
-					'='.$left_join['alias'].'.'.$left_join['using'];
+				$sql_left_join .= ' LEFT JOIN '.$left_join['table'].' '.$left_join['alias'].' ON ';
+				$sql_left_join .= array_key_exists('condition', $left_join)
+					? $left_join['condition']
+					: $sqlParts['left_table']['alias'].'.'.$l_table['key'].'='.
+						$left_join['alias'].'.'.$left_join['using'];
 			}
 
 			// Moving a left table to the end.
@@ -481,53 +480,153 @@ class CApiService {
 	}
 
 	/**
-	 * Modifies the SQL parts to implement all of the output related options.
+	 * Modifies the SQL parts to implement all the output related options.
 	 *
-	 * @param string $tableName
-	 * @param string $tableAlias
+	 * @param string $table_name
+	 * @param string $table_alias
 	 * @param array  $options
-	 * @param array  $sqlParts
+	 * @param array  $sql_parts
 	 *
-	 * @return array		The resulting SQL parts array
+	 * @throws Exception
+	 *
+	 * @return array  The resulting SQL parts array.
 	 */
-	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
-		// If table do not have a primary key, use COUNT(*) to select number of rows.
-		$pk = $this->pk($tableName);
-		$pkFieldId = $this->fieldId($pk, $tableAlias);
+	protected function applyQueryOutputOptions(string $table_name, string $table_alias, array $options,
+			array $sql_parts) {
+		$pk = $this->pk($table_name);
+		$pk_composite = strpos($pk, ',') !== false;
 
-		// count
 		if (array_key_exists('countOutput', $options) && $options['countOutput']
 				&& !$this->requiresPostSqlFiltering($options)) {
-			$sqlParts['select'] = ($pk !== '')
-				? ['COUNT(DISTINCT '.$pkFieldId.') AS rowscount']
+			$has_joins = count($sql_parts['from']) > 1
+				|| (array_key_exists('left_join', $sql_parts) && $sql_parts['left_join']);
+
+			if ($pk_composite && $has_joins) {
+				throw new Exception('Joins with composite primary keys are not supported in this API version.');
+			}
+
+			$sql_parts['select'] = $has_joins
+				? ['COUNT(DISTINCT '.$this->fieldId($pk, $table_alias).') AS rowscount']
 				: ['COUNT(*) AS rowscount'];
 
-			// select columns used by group count
+			// Select columns used by group count.
 			if (array_key_exists('groupCount', $options) && $options['groupCount']) {
-				foreach ($sqlParts['group'] as $fields) {
-					$sqlParts['select'][] = $fields;
+				foreach ($sql_parts['group'] as $fields) {
+					$sql_parts['select'][] = $fields;
 				}
 			}
+			elseif (array_key_exists('groupBy', $options) && $options['groupBy']) {
+				foreach ($options['groupBy'] as $field) {
+					$field = $this->fieldId($field, $table_alias);
+
+					array_unshift($sql_parts['select'], $field);
+					$sql_parts['group'][] = $field;
+				}
+			}
+		}
+		elseif (array_key_exists('groupBy', $options) && $options['groupBy']) {
+			$sql_parts['select'] = [];
+
+			foreach ($options['groupBy'] as $field) {
+				$field = $this->fieldId($field, $table_alias);
+
+				array_unshift($sql_parts['select'], $field);
+				$sql_parts['group'][] = $field;
+			}
 		}
 		// custom output
 		elseif (is_array($options['output'])) {
-			// the pk field must always be included for the API to work properly
-			$sqlParts['select'] = ($pk !== '') ? [$pkFieldId] : [];
+			$sql_parts['select'] = $pk_composite ? [] : [$this->fieldId($pk, $table_alias)];
+
 			foreach ($options['output'] as $field) {
-				if ($this->hasField($field, $tableName)) {
-					$sqlParts['select'][] = $this->fieldId($field, $tableAlias);
+				if ($this->hasField($field, $table_name)) {
+					$sql_parts['select'][] = $this->fieldId($field, $table_alias);
 				}
 			}
 
-			$sqlParts['select'] = array_unique($sqlParts['select']);
+			$sql_parts['select'] = array_unique($sql_parts['select']);
 		}
 		// extended output
 		elseif ($options['output'] == API_OUTPUT_EXTEND) {
 			// TODO: API_OUTPUT_EXTEND must return ONLY the fields from the base table
-			$sqlParts = $this->addQuerySelect($this->fieldId('*', $tableAlias), $sqlParts);
+			$sql_parts = $this->addQuerySelect($this->fieldId('*', $table_alias), $sql_parts);
 		}
 
-		return $sqlParts;
+		return $sql_parts;
+	}
+
+	/**
+	 * Unset the NCLOB fields from output of the given options, if SQL query contains DISTINCT statement. Replace the
+	 * API_OUTPUT_EXTEND value in output of the given options to self::OUTPUT_FIELDS (if defined) or to all fields that
+	 * are specified in table schema.
+	 *
+	 * @param array $options
+	 * @param array $sql_parts
+	 */
+	protected function unsetNclobFieldsFromOutput(array &$options, array $sql_parts): void {
+		if ($options['countOutput'] || !self::dbDistinct($sql_parts)) {
+			return;
+		}
+
+		$schema = $this->getTableSchema();
+		$nclob_fields = [];
+
+		foreach ($schema['fields'] as $field_name => $field) {
+			if ($field['type'] == DB::FIELD_TYPE_NCLOB
+					&& $this->outputIsRequested($field_name, $options['output'])) {
+				$nclob_fields[] = $field_name;
+			}
+		}
+
+		if ($nclob_fields) {
+			if ($options['output'] === API_OUTPUT_EXTEND) {
+				$output = self::OUTPUT_FIELDS
+					? self::OUTPUT_FIELDS
+					: array_keys($schema['fields']);
+			}
+			else {
+				$output = $options['output'];
+			}
+
+			$options['output'] = array_diff($output, $nclob_fields);
+		}
+	}
+
+	/**
+	 * Add NCLOB type fields if there was DISTINCT in query.
+	 *
+	 * @param array $options    Array of query options.
+	 * @param array $result     Query results.
+	 *
+	 * @return array    The result array with added NCLOB fields.
+	 */
+	protected function addNclobFieldValues(array $options, array $result): array {
+		$schema = $this->getTableSchema();
+		$nclob_fields = [];
+
+		foreach ($schema['fields'] as $field_name => $field) {
+			if ($field['type'] == DB::FIELD_TYPE_NCLOB && $this->outputIsRequested($field_name, $options['output'])) {
+				$nclob_fields[] = $field_name;
+			}
+		}
+
+		if (!$nclob_fields) {
+			return $result;
+		}
+
+		$pk = $schema['key'];
+		$options = [
+			'output' => $nclob_fields,
+			'filter' => [$pk => array_keys($result)]
+		];
+
+		$db_items = DBselect(DB::makeSql($this->tableName, $options));
+
+		while ($db_item = DBfetch($db_items)) {
+			$result[$db_item[$pk]] += $db_item;
+		}
+
+		return $result;
 	}
 
 	/**
@@ -564,44 +663,64 @@ class CApiService {
 	}
 
 	/**
-	 * Modifies the SQL parts to implement all of the sorting related options.
+	 * Modifies the SQL parts to implement all the sorting related options.
 	 * Sorting is currently only supported for CApiService::get() methods.
 	 *
-	 * @param string $tableName
-	 * @param string $tableAlias
+	 * @param string $table_name
+	 * @param string $table_alias
 	 * @param array  $options
-	 * @param array  $sqlParts
+	 * @param array  $sql_parts
+	 *
+	 * @throws APIException
 	 *
 	 * @return array
 	 */
-	protected function applyQuerySortOptions($tableName, $tableAlias, array $options, array $sqlParts) {
-		if ($this->sortColumns && !zbx_empty($options['sortfield'])) {
+	protected function applyQuerySortOptions(string $table_name, string $table_alias, array $options,
+			array $sql_parts): array {
+		$count_output = array_key_exists('countOutput', $options) && $options['countOutput'];
+		$group_by = array_key_exists('groupBy', $options) && $options['groupBy'];
+		$aggregate_sort_columns = [];
+
+		if ($count_output && $group_by) {
+			$aggregate_sort_columns[] = 'rowscount';
+		}
+
+		$sort_columns = $group_by ? array_merge($options['groupBy'], $aggregate_sort_columns) : $this->sortColumns;
+
+		if ($sort_columns && !zbx_empty($options['sortfield'])) {
 			$options['sortfield'] = is_array($options['sortfield'])
 				? array_unique($options['sortfield'])
 				: [$options['sortfield']];
 
 			foreach ($options['sortfield'] as $i => $sortfield) {
-				// validate sortfield
-				if (!str_in_array($sortfield, $this->sortColumns)) {
-					throw new APIException(ZBX_API_ERROR_INTERNAL, _s('Sorting by field "%1$s" not allowed.', $sortfield));
+				// Validate sortfield.
+				if (!str_in_array($sortfield, $sort_columns)) {
+					throw new APIException(ZBX_API_ERROR_INTERNAL,
+						_s('Sorting by field "%1$s" not allowed.', $sortfield)
+					);
 				}
 
-				// add sort field to order
+				// Add sort field to order.
 				$sortorder = '';
 				if (is_array($options['sortorder'])) {
 					if (!empty($options['sortorder'][$i])) {
-						$sortorder = ($options['sortorder'][$i] == ZBX_SORT_DOWN) ? ' '.ZBX_SORT_DOWN : '';
+						$sortorder = $options['sortorder'][$i] === ZBX_SORT_DOWN ? ' '.ZBX_SORT_DOWN : '';
 					}
 				}
 				else {
-					$sortorder = ($options['sortorder'] == ZBX_SORT_DOWN) ? ' '.ZBX_SORT_DOWN : '';
+					$sortorder = $options['sortorder'] === ZBX_SORT_DOWN ? ' '.ZBX_SORT_DOWN : '';
 				}
 
-				$sqlParts = $this->applyQuerySortField($sortfield, $sortorder, $tableAlias, $sqlParts);
+				if (in_array($sortfield, $aggregate_sort_columns)) {
+					$sql_parts['order'][$sortfield] = $sortfield.$sortorder;
+				}
+				else {
+					$sql_parts = $this->applyQuerySortField($sortfield, $sortorder, $table_alias, $sql_parts);
+				}
 			}
 		}
 
-		return $sqlParts;
+		return $sql_parts;
 	}
 
 	/**
@@ -872,8 +991,6 @@ class CApiService {
 	/**
 	 * Throws an API exception.
 	 *
-	 * @static
-	 *
 	 * @param int    $code
 	 * @param string $error
 	 */
@@ -888,7 +1005,7 @@ class CApiService {
 	 * @param string $error		error text
 	 */
 	protected function deprecated($error) {
-		trigger_error($error, E_USER_NOTICE);
+		trigger_error($error, E_USER_DEPRECATED);
 	}
 
 	/**
@@ -915,21 +1032,78 @@ class CApiService {
 				continue;
 			}
 
-			zbx_value2array($value);
+			$values = [];
+
+			switch ($tableSchema['fields'][$field]['type']) {
+				case DB::FIELD_TYPE_INT:
+					foreach ((array) $value as $val) {
+						if (!is_int($val) && (!is_string($val) || !preg_match('/^'.ZBX_PREG_INT.'$/', $val))) {
+							continue;
+						}
+
+						if ($val < ZBX_MIN_INT32 || $val > ZBX_MAX_INT32) {
+							continue;
+						}
+
+						$values[] = $val;
+					}
+					break;
+
+				case DB::FIELD_TYPE_ID:
+					foreach ((array) $value as $val) {
+						if (!is_int($val) && (!is_string($val) || !ctype_digit($val))) {
+							continue;
+						}
+
+						if ($val < 0 || bccomp((string) $val, ZBX_DB_MAX_ID) > 0) {
+							continue;
+						}
+
+						$values[] = $val;
+					}
+					break;
+
+				case DB::FIELD_TYPE_UINT:
+					foreach ((array) $value as $val) {
+						if (!is_int($val) && (!is_string($val) || !ctype_digit($val))) {
+							continue;
+						}
+
+						if (bccomp((string) $val, ZBX_MIN_INT64) < 0 || bccomp((string) $val, ZBX_MAX_INT64) > 0) {
+							continue;
+						}
+
+						$values[] = $val;
+					}
+					break;
+
+				case DB::FIELD_TYPE_FLOAT:
+					foreach ((array) $value as $val) {
+						if (!is_numeric($val)) {
+							continue;
+						}
+
+						$values[] = $val;
+					}
+					break;
+
+				default:
+					$values = (array) $value;
+			}
 
 			$fieldName = $this->fieldId($field, $tableShort);
 			switch ($tableSchema['fields'][$field]['type']) {
 				case DB::FIELD_TYPE_ID:
-					$filter[$field] = dbConditionId($fieldName, $value);
+					$filter[$field] = dbConditionId($fieldName, $values);
 					break;
 
 				case DB::FIELD_TYPE_INT:
 				case DB::FIELD_TYPE_UINT:
-					$filter[$field] = dbConditionInt($fieldName, $value);
+					$filter[$field] = dbConditionInt($fieldName, $values);
 					break;
 
 				default:
-					$filter[$field] = dbConditionString($fieldName, $value);
+					$filter[$field] = dbConditionString($fieldName, $values);
 			}
 		}
 
@@ -1134,47 +1308,69 @@ class CApiService {
 	}
 
 	/**
-	 * Add simple audit record.
+	 * Legacy method to add audit log records.
 	 *
-	 * @param int    $action        AUDIT_ACTION_*
-	 * @param int    $resourcetype  AUDIT_RESOURCE_*
-	 * @param string $details
-	 * @param string $userid
-	 * @param string $ip
+	 * @param int   $action        CAudit::ACTION_*
+	 * @param int   $resourcetype  CAudit::RESOURCE_*
+	 * @param array $objects
+	 * @param array $objects_old
 	 */
-	protected function addAuditDetails($action, $resourcetype, $details = '', $userid = null, $ip = null) {
-		if ($userid === null) {
-			$userid = self::$userData['userid'];
-			$ip = self::$userData['userip'];
-		}
-
-		CAudit::addDetails($userid, $ip, $action, $resourcetype, $details);
+	protected function addAuditBulk($action, $resourcetype, array $objects, array $objects_old = null) {
+		CAuditOld::addBulk(self::$userData['userid'], self::$userData['userip'], self::$userData['username'], $action,
+			$resourcetype, $objects, $objects_old
+		);
 	}
 
 	/**
-	 * Add audit records.
+	 * Add audit log records.
 	 *
-	 * @param int    $action        AUDIT_ACTION_*
-	 * @param int    $resourcetype  AUDIT_RESOURCE_*
-	 * @param array  $objects
-	 * @param array  $objects_old
+	 * @param int   $action       CAudit::ACTION_*
+	 * @param int   $resource     CAudit::RESOURCE_*
+	 * @param array $objects      (optional)
+	 * @param array $objects_old  (optional)
 	 */
-	protected function addAuditBulk($action, $resourcetype, array $objects, array $objects_old = null) {
-		CAudit::addBulk(self::$userData['userid'], self::$userData['userip'], $action, $resourcetype, $objects,
-			$objects_old
+	protected static function addAuditLog(int $action, int $resource, array $objects = [],
+			array $objects_old = []): void {
+		CAudit::log(self::$userData['userid'], self::$userData['userip'], self::$userData['username'], $action,
+			$resource, $objects, $objects_old
 		);
 	}
 
+	/**
+	 * Add audit log records on behalf of the given user.
+	 *
+	 * @param string|null $userid
+	 * @param string      $ip
+	 * @param string      $username
+	 * @param int         $action       CAudit::ACTION_*
+	 * @param int         $resource     CAudit::RESOURCE_*
+	 * @param array       $objects      (optional)
+	 * @param array       $objects_old  (optional)
+	 */
+	protected static function addAuditLogByUser(?string $userid, string $ip, string $username, int $action,
+			int $resource, array $objects = [], array $objects_old = []): void {
+		CAudit::log($userid, $ip, $username, $action, $resource, $objects, $objects_old);
+	}
+
 	/**
 	 * Check access to specific access rule.
 	 *
-	 * @static
-	 *
 	 * @param string $rule_name  Rule name.
 	 *
 	 * @return bool  Returns true if user has access to specified rule, and false otherwise.
+	 *
+	 * @throws Exception
 	 */
 	protected static function checkAccess(string $rule_name): bool {
 		return (self::$userData && CRoleHelper::checkAccess($rule_name, self::$userData['roleid']));
 	}
+
+	/**
+	 * Return user session ID or user API token.
+	 *
+	 * @return string
+	 */
+	public static function getAuthIdentifier(): string {
+		return array_key_exists('sessionid', self::$userData) ? self::$userData['sessionid'] : self::$userData['token'];
+	}
 }
