@@ -213,8 +213,9 @@ function check_type(&$field, $flags, &$var, $type, $caption = null) {
 			$error = true;
 			$message = _s('Field "%1$s" is not string.', $caption);
 		}
-		elseif (mb_check_encoding($var, 'UTF-8') !== true) {
+		elseif (zbx_mb_check_encoding($var, 'UTF-8') !== true) {
 			error(_s('Field "%1$s" is not correct: %2$s.', $caption, _('invalid byte sequence in UTF-8')));
+
 			return ZBX_VALID_ERROR;
 		}
 	}
