@@ -99,6 +99,13 @@ class CSVGHoneycomb {
 	 */
 	#cells_max_count;
 
+	/**
+	 * Limit for maximum number of cells to display in the widget.
+	 *
+	 * @type {number}
+	 */
+	#cells_max_count_limit = 1000;
+
 	/**
 	 * Width of cell (inner radius).
 	 * It is large number because SVG works more precise that way (later it will be scaled according to widget size).
@@ -306,14 +313,10 @@ class CSVGHoneycomb {
 			};
 		};
 
-		const cell_min_width = CSVGHoneycomb.CELL_WIDTH_MIN;
-		const cell_min_height = CSVGHoneycomb.CELL_WIDTH_MIN / Math.sqrt(3) * 2;
-
-		const max_rows = Math.floor((this.#height - cell_min_height) / (cell_min_height * .75)) + 1;
-		const max_columns = Math.floor((this.#width - (max_rows > 1 ? cell_min_width / 2 : 0)) / cell_min_width);
+		const {max_rows, max_columns} = CSVGHoneycomb.getContainerMaxParams({width: this.#width, height: this.#height});
 
 		this.#cells_max_count = this.#cells_data !== null
-			? Math.min(this.#cells_data.length, max_rows * max_columns)
+			? Math.min(this.#cells_max_count_limit, this.#cells_data.length, max_rows * max_columns)
 			: 0;
 
 		const rows = Math.max(1, Math.min(max_rows, this.#cells_max_count,
@@ -997,4 +1000,22 @@ class CSVGHoneycomb {
 	static #getUniqueId() {
 		return `CSVGHoneycomb-${this.ID_COUNTER++}`;
 	}
+
+	/**
+	 * Get honeycomb container max row and max column count.
+	 *
+	 * @param {number} width
+	 * @param {number} height
+	 *
+	 * @returns {{max_rows: number, max_columns: number}}
+	 */
+	static getContainerMaxParams({width, height}) {
+		const cell_min_width = CSVGHoneycomb.CELL_WIDTH_MIN;
+		const cell_min_height = CSVGHoneycomb.CELL_WIDTH_MIN / Math.sqrt(3) * 2;
+
+		const max_rows = Math.floor((height - cell_min_height) / (cell_min_height * .75)) + 1;
+		const max_columns = Math.floor((width - (max_rows > 1 ? cell_min_width / 2 : 0)) / cell_min_width);
+
+		return {max_rows, max_columns};
+	}
 }
